/* --------------------------------------------------------- */
/*	osziFOX ʐMW[									 */
/* ========================================================= */
/*	(C)2004 t.hara											 */
/* --------------------------------------------------------- */

#include <tchar.h>
#include <windows.h>
#include <process.h>
#include "stdafx.h"
#include "OsziFoxConnect.h"

#define	FOX_TIME_OUT	500

/* --------------------------------------------------------- */
CFoxConnect::CFoxConnect() {

	hPort = NULL;
	hDoReceive = NULL;
	hDoReceive = NULL;
	hThread = NULL;
	hReceiveNotify = NULL;
	PortNumber = -1;
	fThreadDrive = FALSE;
	::ZeroMemory( &Data, sizeof(Data) );
	::InitializeCriticalSection( &HandleSaveCs );
}

/* --------------------------------------------------------- */
CFoxConnect::~CFoxConnect() {

	Terminate();
	::DeleteCriticalSection( &HandleSaveCs );
}

/* --------------------------------------------------------- */
BOOL CFoxConnect::WaitSyncSignal( void ) {
	BYTE Signal;
	DWORD Bytes;

	if( !ReadFile( hPort, &Signal, 1, &Bytes, NULL ) ) return FALSE;
	if( Bytes == 0 ) return FALSE;
	return( (Signal & 0x7C) == 0x7C );
}

/* --------------------------------------------------------- */
void CFoxConnect::Receive( void ) {
	BYTE Signal, ReceiveData[ sizeof(Data) ];
	DWORD Bytes;
	int Size;

	while( fThreadDrive ) {
		::WaitForSingleObject( hDoReceive, INFINITE );
		if( !fThreadDrive ) break;
		::EnterCriticalSection( &HandleSaveCs );
		if( !WaitSyncSignal() ) {
			::LeaveCriticalSection( &HandleSaveCs );
			continue;
		}
		::LeaveCriticalSection( &HandleSaveCs );
		/* M */
		while( fThreadDrive ) {
			if( ::WaitForSingleObject( hDoReceive, 0 ) == WAIT_TIMEOUT ) {
				break;
			}
			Size = 0;
			::EnterCriticalSection( &HandleSaveCs );
			while( fThreadDrive ) {
				if( !ReadFile( hPort, &Signal, 1, &Bytes, NULL ) ) {
					break;
				}
				if( Bytes == 0 ) {
					break;
				}
				if( (Signal & 0x7C) == 0x7C ) {
					break;
				}
				if( Size >= sizeof(ReceiveData) ) {
					break;
				}
				ReceiveData[ Size ] = Signal;
				Size++;
			}
			/* ` */
			if( Size < 133 ) {
				::CopyMemory( Data, ReceiveData, 4 );
			}
			else if( Size < 137 ) {
				::CopyMemory( Data + 4, ReceiveData, 133 );
			}
			else {
				::CopyMemory( Data, ReceiveData, 137 );
			}
			::LeaveCriticalSection( &HandleSaveCs );
			::SetEvent( hReceiveNotify );
		}
	}
}

/* --------------------------------------------------------- */
unsigned int CALLBACK CFoxConnect::ReceiveThread( void *pThis ) {

	((CFoxConnect*)pThis)->Receive();
	::_endthreadex( 0 );
	return 0;
}

/* --------------------------------------------------------- */
void CFoxConnect::Initialize( void ) {
	unsigned int ThreadId;

	Terminate();
	hDoReceive = ::CreateEvent( NULL, TRUE, FALSE, NULL );
	if( hDoReceive == NULL ) {
		throw( (TCHAR*) _T("Cannot create a event.") );
	}
	hReceiveNotify = ::CreateEvent( NULL, FALSE, FALSE, NULL );
	if( hReceiveNotify == NULL ) {
		throw( (TCHAR*) _T("Cannot create a event.") );
	}
	fThreadDrive = TRUE;
	hThread = (HANDLE) ::_beginthreadex( NULL, 0, ReceiveThread, this, 0, &ThreadId );
	if( hThread == INVALID_HANDLE_VALUE ) {
		throw( (TCHAR*) _T("Cannot start a thread for receive.") );
	}
}

/* --------------------------------------------------------- */
void CFoxConnect::Terminate( void ) {

	if( hPort != NULL ) {
		::CloseHandle( hPort );
		hPort = NULL;
	}
	if( hThread != NULL ) {
		fThreadDrive = FALSE;
		::SetEvent( hDoReceive );
		if( ::WaitForSingleObject( hThread, 3000 ) == WAIT_TIMEOUT ) {
			::TerminateThread( hThread, 0 );
		}
		::CloseHandle( hThread );
		hThread = NULL;
	}
	if( hDoReceive != NULL ) {
		::CloseHandle( hDoReceive );
		hDoReceive = NULL;
	}
	if( hReceiveNotify != NULL ) {
		::CloseHandle( hReceiveNotify );
		hReceiveNotify = NULL;
	}
}

/* --------------------------------------------------------- */
BOOL CFoxConnect::SetPortNumber( int _PortNumber ) {
	TCHAR sPortName[24];

	::EnterCriticalSection( &HandleSaveCs );
	if( hPort != NULL ) {
		::CloseHandle( hPort );
		PortNumber = -1;
	}
	::wsprintf( sPortName, _T("\\\\.\\COM%d"), _PortNumber+1 );
	hPort = ::CreateFile( sPortName, GENERIC_READ, 0, NULL,
			OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
	if( hPort == INVALID_HANDLE_VALUE ) {
		hPort = NULL;
		::LeaveCriticalSection( &HandleSaveCs );
		return FALSE;
	}
	PortNumber = _PortNumber;

	COMMTIMEOUTS CommTimeouts;
	CommTimeouts.ReadIntervalTimeout = FOX_TIME_OUT;
	CommTimeouts.ReadTotalTimeoutConstant = FOX_TIME_OUT;
	CommTimeouts.ReadTotalTimeoutMultiplier = FOX_TIME_OUT;
	CommTimeouts.WriteTotalTimeoutConstant = FOX_TIME_OUT;
	CommTimeouts.WriteTotalTimeoutMultiplier = FOX_TIME_OUT;
	::SetCommTimeouts( hPort, &CommTimeouts );

	DCB Dcb;
	::ZeroMemory( &Dcb, sizeof(Dcb) );
	Dcb.fBinary = TRUE;
	Dcb.BaudRate = CBR_19200;
	Dcb.ByteSize = 7;
	Dcb.Parity = NOPARITY;
	Dcb.StopBits = ONESTOPBIT;
	::SetCommState( hPort, &Dcb );
	::LeaveCriticalSection( &HandleSaveCs );
	return TRUE;
}

/* --------------------------------------------------------- */
int CFoxConnect::GetPortNumber( void ) {

	return PortNumber;
}

/* --------------------------------------------------------- */
void CFoxConnect::GetData( BYTE *pData ) {

	::EnterCriticalSection( &HandleSaveCs );
	::CopyMemory( pData, Data, sizeof(Data) );
	::LeaveCriticalSection( &HandleSaveCs );
}

/* --------------------------------------------------------- */
void CFoxConnect::Start( void ) {

	::SetEvent( hDoReceive );
}

/* --------------------------------------------------------- */
void CFoxConnect::Stop( void ) {

	::ResetEvent( hDoReceive );
}

/* --------------------------------------------------------- */
BOOL CFoxConnect::IsStart( void ) {

	return( ::WaitForSingleObject( hDoReceive, 0 ) == WAIT_OBJECT_0 );
}

/* --------------------------------------------------------- */
CFoxData::CFoxData() {

	::ZeroMemory( Data, sizeof(Data) );
}

/* --------------------------------------------------------- */
void CFoxData::SetData( CFoxConnect *pConnect ) {

	pConnect->GetData( Data );
}

/* --------------------------------------------------------- */
void CFoxData::SetData( BYTE *pData ) {

	::CopyMemory( Data, pData, sizeof(Data) );
}

/* --------------------------------------------------------- */
int CFoxData::GetVoltage( void ) {
	static int cVoltage[] = {
		1, 10, 100, 0,
	};
	return cVoltage[ (Data[0] >> 2) & 3 ];
}

/* --------------------------------------------------------- */
int CFoxData::GetTimeBase( void ) {
	static int cTimeBase[] = {
		50, 100, 500, 1000, 5000, 10000,
		50000, 100000, 500000, 1000000, 2000000,
		0, 0, 0, 0, 0,
	};
	return cTimeBase[ Data[1] & 15 ];
}

/* --------------------------------------------------------- */
LPCTSTR CFoxData::GetCoupling( void ) {
	static LPCTSTR csCoupling[] = {
		_T("GND"), _T("AC"), _T("DC"), _T("N/A"),
	};
	return csCoupling[ (Data[0] >> 4) & 3 ];
}

/* --------------------------------------------------------- */
LPCTSTR CFoxData::GetTriggerType( void ) {

	int Type = (Data[2] >> 2) & 15;
	switch( Type ) {
	case 8:
		return _T("+INTERN");
	case 4:
		return _T("-INTERN");
	case 2:
		return _T("+EXTERN");
	case 1:
		return _T("-EXTERN");
	default:
		break;
	}
	return _T("FREE RUN");
}

/* --------------------------------------------------------- */
float CFoxData::GetTriggerLevel( void ) {

	int Type = (Data[3] >> 2) & 15;
	switch( Type ) {
	case 0x00:
		return -0.5f;
	case 0x10:
		return -0.3f;
	case 0x08:
		return -0.1f;
	case 0x04:
		return  0.1f;
	case 0x02:
		return  0.3f;
	case 0x01:
		return  0.5f;
	default:
		break;
	}
	return 0.0f;
}
