/* --------------------------------------------------------- */
/*	osziFOX graph area										 */
/* ========================================================= */
/*	(C)2004 t.hara											 */
/* --------------------------------------------------------- */

#include "stdafx.h"
#include "resource.h"

#include "osziFoxViewerView.h"

/* --------------------------------------------------------- */
COsziFoxViewerView::COsziFoxViewerView() {

	pBackBrush = NULL;
	pGraphPen = NULL;
	pGraphGridPen = NULL;
	pGraphGridDotPen = NULL;
	pTextBrush = NULL;
	pTextFont = NULL;
	pBackScreen = NULL;
	fDrive = TRUE;
	hUpdateThread = NULL;
	hReceiveEvent = NULL;
	
	::InitializeCriticalSection( &BackScreenCs );
}

/* --------------------------------------------------------- */
COsziFoxViewerView::~COsziFoxViewerView() {

	::DeleteCriticalSection( &BackScreenCs );
}

/* --------------------------------------------------------- */
void COsziFoxViewerView::Update( void ) {

	while( fDrive ) {
		if( ::WaitForSingleObject( hReceiveEvent, INFINITE ) != WAIT_OBJECT_0 ) {
			::Sleep( 100 );
		}
		else {
			Wave.SetData( pConnect );
			DrawGraph();
			InvalidateRect( NULL, FALSE );
		}
	}
}

/* --------------------------------------------------------- */
unsigned int CALLBACK COsziFoxViewerView::UpdateCheck( void *pThis ) {
	
	((COsziFoxViewerView*)pThis)->Update();
	_endthreadex( 0 );
	return 0;
}

/* --------------------------------------------------------- */
BOOL COsziFoxViewerView::PreTranslateMessage(MSG* pMsg) {
	pMsg;
	return FALSE;
}

/* --------------------------------------------------------- */
LRESULT COsziFoxViewerView::OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/) {
	unsigned int ThreadId;

	SetClassLong( this->m_hWnd, GCL_HBRBACKGROUND, (LONG)(HBRUSH) NULL );

	/* create GDI+ resources */
	RECT Rect;
	GetClientRect( &Rect );
	Graphics g( m_hWnd );
	pBackScreen = new Bitmap( Rect.right, Rect.bottom, &g );

	float DashPattern[] = { 4.f, 4.f };
	Color BackColor( 0, 0, 64 );
	Color GraphColor( 100, 255, 100 );
	Color GraphGridColor( 0, 80, 180 );
	pBackBrush = new SolidBrush( BackColor );
	pGraphPen = new Pen( GraphColor );
	pGraphGridPen = new Pen( GraphGridColor );
	pGraphGridDotPen = new Pen( GraphGridColor );
	pGraphGridDotPen->SetDashPattern( DashPattern, sizeof(DashPattern)/sizeof(float) );

	Color TextColor( 255, 255, 200 );
	pTextBrush = new SolidBrush( TextColor );
	pTextFont = new Font( L"Times New Roman", 16 );

	hReceiveEvent = pConnect->GetSyncObject();
	hUpdateThread = (HANDLE) ::_beginthreadex( NULL, 0, UpdateCheck, this, 0, &ThreadId );
	if( hUpdateThread == INVALID_HANDLE_VALUE ) {
		throw( (TCHAR*) _T("Cannot start a thread for receive.") );
	}

	return 0;
}

/* --------------------------------------------------------- */
LRESULT COsziFoxViewerView::OnPaint(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
	CPaintDC dc( m_hWnd );
	Graphics g( dc.m_hDC );

	g.DrawImage( pBackScreen, 0, 0 );
	return 0;
}

/* --------------------------------------------------------- */
LRESULT COsziFoxViewerView::OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {

	if( hUpdateThread != NULL ) {
		fDrive = FALSE;
		if( hReceiveEvent != NULL ) {
			::SetEvent( hReceiveEvent );
		}
		if( ::WaitForSingleObject( hUpdateThread, 3000 ) == WAIT_TIMEOUT ) {
			::TerminateThread( hUpdateThread, 0 );
		}
		::CloseHandle( hUpdateThread );
	}

	/* delete GDI+ resources */
	delete pTextFont;			pTextFont = NULL;
	delete pTextBrush;			pTextBrush = NULL;
	delete pGraphGridDotPen;	pGraphGridDotPen = NULL;
	delete pGraphGridPen;		pGraphGridPen = NULL;
	delete pGraphPen;			pGraphPen = NULL;
	delete pBackScreen;			pBackScreen = NULL;
	delete pBackBrush;			pBackBrush = NULL;
	return 0;
}

/* --------------------------------------------------------- */
void COsziFoxViewerView::DrawGraph( void ) {
	int i;

	::EnterCriticalSection( &BackScreenCs );
	Graphics g( pBackScreen );

	g.SetSmoothingMode( SmoothingModeHighQuality );

	/* aspect ratio */
	int Width = (int) pBackScreen->GetWidth();
	int Height = (int) pBackScreen->GetHeight();
	if( Width * 4 > Height * 3 ) {
		Width = Height * 4 / 3;
	}
	else {
		Height = Width * 3 / 4;
	}
	g.FillRectangle( pBackBrush, 0, 0, pBackScreen->GetWidth(), pBackScreen->GetHeight() );
	g.ScaleTransform( Width / (REAL)640, Height / (REAL)480 );

	/* drawing */
	g.DrawLine( pGraphGridPen, 8,     8,     8+440, 8     );
	g.DrawLine( pGraphGridPen, 8+440, 8,     8+440, 8+440 );
	g.DrawLine( pGraphGridPen, 8+440, 8+440, 8,     8+440 );
	g.DrawLine( pGraphGridPen, 8,     8+440, 8    , 8     );

	for( i = 55; i < 440; i += 55 ) {
		g.DrawLine( pGraphGridDotPen, 8+i, 8,   8+i,   8+440 );
		g.DrawLine( pGraphGridDotPen, 8,   8+i, 8+440, 8+i   );
	}

	int ly = (8+440) - (Wave.Data[ CFoxConnect::WAVE_OFFSET ] & 63) * 440 / 64;;
	int y;
	for( i = 1; i < CFoxConnect::WAVE_SIZE; i++ ) {
		y = (8+440) - (Wave.Data[ CFoxConnect::WAVE_OFFSET + i ] & 63) * 440 / 64;
		g.DrawLine( pGraphPen, 8 + ((i-1) * 440) / CFoxConnect::WAVE_SIZE, ly,
				8 + (i * 440) / CFoxConnect::WAVE_SIZE, y );
		ly = y;
	}

	CString Message;
	PointF pt;

	pt.X = 460;
	pt.Y = 8+24*0;
	if( pConnect->IsStart() ) {
		Message.Format( _T("Connect: COM%d"), pConnect->GetPortNumber() + 1 );
	}
	else {
		Message = _T("Connect: ڑ");
	}
	g.DrawString( Message, Message.GetLength(), pTextFont, pt, pTextBrush );

	pt.X = 460;
	pt.Y = 8+24*1;
	Message.Format( _T("Coupling: %s"), Wave.GetCoupling() );
	g.DrawString( Message, Message.GetLength(), pTextFont, pt, pTextBrush );

	pt.X = 460;
	pt.Y = 8+24*2;
	Message.Format( _T("Trigger Type: %s"), Wave.GetTriggerType() );
	g.DrawString( Message, Message.GetLength(), pTextFont, pt, pTextBrush );

	pt.X = 460;
	pt.Y = 8+24*3;
	Message.Format( _T("Trigger Level: %1.1f"), Wave.GetTriggerLevel() );
	g.DrawString( Message, Message.GetLength(), pTextFont, pt, pTextBrush );

	pt.X = 460;
	pt.Y = 8+24*4;
	Message.Format( _T("Time Base: %dnsec"), Wave.GetTimeBase() );
	g.DrawString( Message, Message.GetLength(), pTextFont, pt, pTextBrush );

	pt.X = 460;
	pt.Y = 8+24*5;
	Message.Format( _T("Voltage: %dV"), Wave.GetVoltage() );
	g.DrawString( Message, Message.GetLength(), pTextFont, pt, pTextBrush );

	pt.X = 460;
	pt.Y = 8+24*6;
	Message.Format( _T("V-Div: %2.3fV"), (double)Wave.GetVoltage() / 8. );
	g.DrawString( Message, Message.GetLength(), pTextFont, pt, pTextBrush );

	pt.X = 460;
	pt.Y = 8+24*7;
	Message.Format( _T("H-Div: %.3fnsec"), (double)Wave.GetTimeBase() * 128. / 8. );
	g.DrawString( Message, Message.GetLength(), pTextFont, pt, pTextBrush );

	Graphics gWnd( m_hWnd );
	gWnd.DrawImage( pBackScreen, 0, 0 );

	::LeaveCriticalSection( &BackScreenCs );
}

/* --------------------------------------------------------- */
LRESULT COsziFoxViewerView::OnSize(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {

	if( pBackScreen == NULL ) return 0;	/* WM_DESTROY ̌ WM_SIZE ΍ */
	::EnterCriticalSection( &BackScreenCs );
	delete pBackScreen;

	RECT Rect;
	GetClientRect( &Rect );
	Graphics g( m_hWnd );
	pBackScreen = new Bitmap( Rect.right, Rect.bottom, &g );
	::LeaveCriticalSection( &BackScreenCs );

	DrawGraph();
	InvalidateRect( NULL, FALSE );
	return 0;
}
